function [y,info_bits] = Array_TxModElectroabsorp_RZ(UnmodulatedPower,Linewidth,...
    ModulationIndex,ChirpFactor,RZRatio,PreSpaces,PostSpaces,RiseTime,LaserNumber,LaserSpacing)
%
% Usage: [y,info_bits] = Array_TxModElectroabsorp_RZ(UnmodulatedPower,Linewidth,...
%    ModulationIndex,ChirpFactor,RZRatio,PreSpaces,PostSpaces,RiseTime,LaserNumber,LaserSpacing)
%
% Purpose = This module is to simulate an array of externally RZ modulated laser transmitter
% with CW laser and ModulatorElectroabsorp.
%          
% Array_TxModElectroabsorp_RZ = N_ch Array of (info_bits + RZCoder 
%                     + RiseTimeAdjust + LaserCW + ModulatorElectroabsorp)
%
% Input = Not required.
% Output1(y) = N channels of optical modulated signals separated by Laser spacing. (Size = [N x DataLength]) 
% Output2(info_bits) = Info_bits (Size = [N x Bits]) for BER calculation at the receiver.
%
%=== Physical Parameters with Default Values=======
% UnmodulatedPower= 1e-3 (default);  % [W] CW Laser 
% Linewidth= 10e6 (default);     %[Hz] CW Laser
%
% ModulationIndex = 0.9 (default); %range [0,1] (Extinction ratio control) ModulatorElectroabsorp
% ChirpFactor = 0 (default); %range ]-inf,inf[ ModulatorElectroabsorp
%
% RZRatio = 0.5 (default) %[0,1]  RZCoder
%
% PreSpaces = 2 (default); % number of leading zero bits, infobits_PRBS
% PostSpaces = 2 (default); % number of trailing zero bits, infobits_PRBS
%
% RiseTime = 1/16/BitRate (default);% RiseTimeAdjust  [sec]
%
% LaserNumber = ChannelNumber (default)
% LaserSpacing = ChannelSpacing (default)
%
% Note) Extinction Ratio [dB] = -10log10(1-ModulationIndex)
%
%------------------------------------------------------------
% $ Revision: 1.1 $ Date: 2011/02/22 by Jong-Hyung Lee  
%------------------------------------------------------------
%
%====================================================================================
global t_range BitRate ChannelNumber ChannelSpacing

if nargin <10
    LaserSpacing = ChannelSpacing;
end
if nargin <9
    LaserNumber = ChannelNumber;
end
if nargin <8
    RiseTime = 1/16/BitRate;
end
if nargin <7
    PostSpaces = 2;   
end    
if nargin<6
    PreSpaces = 2;
end
if nargin<5,
    RZRatio = 0.5;
end
if nargin<4,
   ChirpFactor = 0;
end
if nargin<3,
   ModulationIndex = 0.9;
end
if nargin<2,
   Linewidth= 10e6;
end
if nargin==0,
   UnmodulatedPower= 1e-3;
end

%===================================================================
if LaserNumber == 1
   
   [y,info_bits] = TxModElectroabsorp_RZ(UnmodulatedPower,Linewidth,ModulationIndex,ChirpFactor,...
                         RZRatio,PreSpaces,PostSpaces,RiseTime);
              
          
 elseif length(LaserSpacing) == 1   ;  % Equelly spaced case
   
   
	for k = 1:LaserNumber
   
   	  	 df_carrier(k,:) = exp(1i*2*pi*t_range*LaserSpacing*(k - (LaserNumber+1)/2));
         [x,info_bits(k,:)] = TxModElectroabsorp_RZ(UnmodulatedPower,Linewidth,ModulationIndex,ChirpFactor,...
                         RZRatio,PreSpaces,PostSpaces,RiseTime);
         y(k,:) = df_carrier(k,:).*x;
                 
    end
   
   
elseif length(LaserSpacing) > 1   ;  % Unequally spaced case
   
    for k = 1:LaserNumber ;
      	      
         df_carrier(k,:) = exp(1i*2*pi*t_range*LaserSpacing(k));
         [x,info_bits(k,:)] = TxModElectroabsorp_RZ(UnmodulatedPower,Linewidth,ModulationIndex,ChirpFactor,...
                         RZRatio,PreSpaces,PostSpaces,RiseTime);
         y(k,:) = df_carrier(k,:).*x;
           	   	       
	 end
   
end

