function y = EDFA_power(Optical_input,LockedTarget,IncludeNoise,NFdB)
%
% Usage: y = EDFA_power(Optical_input,LockedTarget,IncludeNoise,NFdB)
% 
% Purpose = This module is to simulate an ideal EDFA (wavelength independent gain and noise figure).
%          with a constant output power.
%
% Input(Optical_input) = Optical signal
% Output(y) = Amplified optical signal
%
%=== Physical Parameters=======
%
% LockedTarget = Target Output Power[dBm]  
%              = 20 (default)
% IncludeNoise = 'Yes' or 'No'
%              = 'Yes' (default)
% NFdB = Noise figure in [dB]
%      = 5  (default)
%
% Note1) Noise power is ignored in consideration of EDFA output power. (variable to Noise BW)
% Note2) ASE noise modeled by a complex Gaussain process
% Note3) Single polarization assumed.
%
%----------------------------------------------------------
% $ Revision: 1.1 $ Date: 2011/02/22 by Jong-Hyung Lee  
%------------------------------------------------------------
%
%====================================================================================
global SampleRate DataLength Ref_Frequency

if nargin<4, NFdB=5; end
if nargin<3, IncludeNoise = 'Yes'; end
if nargin<2, LockedTarget = 20; end

Pin_dBm = PowerMeterdBm(Optical_input);
G_edfa = 10^((LockedTarget-Pin_dBm)/10); % Unsaturated(constant) EDFA gain in linear scale

%--- include ASE noise or not -------------------------------

if strcmp(IncludeNoise, 'No') == 1,

	y = sqrt(G_edfa)*Optical_input;

elseif strcmp(IncludeNoise, 'Yes') == 1,

   %---------- ASE noise ----------------------------------------
   h_plank = 6.6256e-34; %[W sec^2] Plank's constant
   P_den_ase_r = .25*10^(NFdB/10)*(G_edfa-1)*h_plank*Ref_Frequency; %[W/Hz]
   E_ase_r =sqrt(P_den_ase_r*SampleRate)*randn(1,DataLength);
   P_den_ase_i = P_den_ase_r;
   E_ase_i =sqrt(P_den_ase_i*SampleRate)*randn(1,DataLength);

   E_ase = E_ase_r + 1i*E_ase_i; % Complex Gaussian Process
   %-------------------------------------------------------------
	E_sig_out = sqrt(G_edfa)*Optical_input;
	y = E_sig_out + E_ase ;
    
else
   % Display Error Message and Stop
   error('IncludeNoise allows only Yes or No. Please check your spelling.') 

end
