function y = OpticalSpectrumAnalyzer(InOptical,Res_FilterBW,Res_Filter)
%
% Usage: y = OpticalSpectrumAnalyzer(InOptical,Res_FilterBW,Res_Filter)(;) 
% Note) y[W] = Calculated power of InOptical. Use semicolon(;) to suppress it.
%          
% Purpose = This module is to plot optical spectrum.
%
% Input(InOptical) = Optical signal
% Output = Optical Spectrum (Resolution Filter Shape & BW can be controlled.)
%
% === Parameters =====
%
% Res_FilterBW = Resolution filter bandwidth in Hz
%               =  BitRate/4  (default)
%
% Res_Filter = Resolution filter shape selection (Ex.) 'Rectangular','Hanning','Hamming','Gauss' etc)
%                 = 'Rectangular' (default)
%
% Note1) Max Resolution = 1/TimeWindow (when Res_FilterBW = BitRate/Bits, that is, nfft = DataLength)
% Note2) Resulting spectral density length = 'nfft' since input signal is complex
% Note3) df = Res_FilterBW if rel_width = 2^n form 
%----------------------------------------------------------
% $ Revision: 1.2 $ Date: 2012/04/01 by Jong-Hyung Lee  
%------------------------------------------------------------
% 1.1 := revised to rearrange the input arguments of 'pwelch' function according to version 6.5
% 1.11 := use 'psd' instead of 'pwelch' function ('psd' is faster and the
% order of input arguments remains same for ver 6.x. However, the
% calculated psd should be normalized by 'SampleRate' to give a meaningful number.)
% 1.2  := For Matlab ver 7.0, use 'pwelch' instead of 'psd'.
%         Total power calulated in frequency domain using 'psd' =  df*sum(P_density)/SampleRate
%         Total power calulated in frequency domain using 'pwelch' =  df*sum(P_density)
%         Total power should be equal to the value calculated in time domain using 'PowerMeter'. 
%
%====================================================================================================================
global SampleRate BitRate ChannelNumber

if isreal(InOptical) == 1  ; % 
   error('Input signal is NOT complex!');
end

[a,b] = size(InOptical);

if a == 1  ; % single channel case
   Input_Optical_Fields = InOptical;
elseif a== ChannelNumber; % multi channel case
   Input_Optical_Fields = sum(InOptical);
else
     error('Please check the size of input optical signal.')
end

if nargin<3,
   Res_Filter = 'Rectangular';
end

if nargin<2,
  Res_FilterBW = BitRate/4;
end

%-----------------------------------------------------------
rel_width = SampleRate/Res_FilterBW;
nfft = round(rel_width); %'round' makes 'rel_width' an interger. Ideally, it should be a 2^n form to speed up.
% Resolution of PSD estimator will be 'SampleRate/nfft'

%==========================================================================

if strcmp(Res_Filter,'Rectangular') == 1
   window_s = rectwin(nfft);
   
elseif strcmp(Res_Filter,'Hanning') == 1   
   window_s = hanning(nfft);
   
elseif strcmp(Res_Filter,'Hamming') == 1
   window_s = hamming(nfft);   

elseif strcmp(Res_Filter,'Gauss') == 1 
   window_s = gausswin(nfft);   
 
else
 
    error('Please check your spelling.') 
                          
end

   % Overlap Percentange = 50
   [P_dens,f] = pwelch(Input_Optical_Fields,window_s,50, nfft,SampleRate);% 
   
   % df = Res_FilterBW if rel_width = 2^n form 
   % f= 0:df:(SampleRate-df),  	
   % new_f = -SampleRate/2: df : (SampleRate-df);  
   % length(f) = length(P_dens) = nfft
   
   df=f(2)-f(1);
   new_f = f-.5*SampleRate; %This is to make f from negative to positive range,  
   P_per_df = 10*log10(fftshift(df*P_dens))+30; % power within each df, +30 is for dBm unit
   P_max = max(P_per_df); P_min = min(P_per_df);
   x_text = (max(new_f)-min(new_f))/5; y_d = P_max - P_min; 
   
   figure 
   plot(new_f,P_per_df)
   xlabel('Relative frequency [Hz]');ylabel('Optical Power[dBm]')
   ResolutionBW_display = sprintf('%0.4g',Res_FilterBW);
   text(x_text,P_max-0.2*y_d,['ResolutionBW =  ' num2str(ResolutionBW_display) '[Hz]'])
   title('Optical Spectrum')

    y = df*sum(P_dens);% Power calculated in [W]
