function [BER_min,i_th_opt,RelativeSampleTime_opt,RelativeThreshold_opt,Q_max] = QnBER_ISI_Optimum(InputElectrical,info_bit,EyeDiagramOn,BER3D)
%
% Usage :
% [BER_min,i_th_opt,RelativeSampleTime_opt,RelativeThreshold_opt,Q_max] = QnBER_ISI_Optimum(InputElectrical,info_bit,EyeDiagramOn,BER3D)
%
% Purpose = This module is to find optimum threshold (decision level) & optimum Sampling instant
%           such that BER = BERmin (& Q = Qmax) based on Gaussian assumption. (WITH considering ISI effects)
%
% Input1(InputElectrical) = Single channel signal (photo-detected electrical signal), Size = [1 x DataLength]
% Input2 (info_bit) = Bit sequence sent at the transmitter
%
% Output1 = Optimum (min) BER at the optimum decision level and SampleTime
% Output2 = Optimum threshold value, i_th_opt = ms0 + RelativeThreshold_opt*(ms1 - ms0)
%           where ms0 &ms1 are obtained at the optimum Sampling time
% Output3 = Optimum Relative Sample Time to Bit Period
% Output4 = Optimum Relative Threshold Value
% Output5 = Optimum (max) Q-value at the optimum decision level and SampleTime  
%
%==== Simulation Parameters ===============
%  EyeDiagramOn = 'Yes' ( Default, Plot Eye Diagram with Sampling instant and decision level using 'EyePlotter2')
%  BER3D = 'Yes' (Default, Plot 3D BER mesh curve.)
%  
%  Note1) Q-value is defined by the relationship, BER = (1/2)*erfc(Q/sqrt(2)) ~ exp(-Q^2/2)/(Q*sqrt(2*pi).
%  Note2) Neighboring one bit at ecah sides has been included to consider ISI effect. 
%

%----------------------------------------------------------
% $ Revision: 1.0 $ Date: 2008/04/29 by Jong-Hyung Lee  
%----------------------------------------------------------
%
%======================================================================================================================
global BitRate SampleRate

if nargin<4, BER3D = 'Yes'; end
if nargin<3, EyeDiagramOn = 'Yes'; end

% 1. Initial sweep of SampleTime & Threshold(Relative)

SampleTime = 0.1:0.1:0.9;
Threshold = 0.2:0.1:0.8;

for p = 1:length(Threshold);
    
    for q = 1:length(SampleTime);
        
       BER1(p,q) = qnber_ISI(InputElectrical,info_bit,SampleTime(q),Threshold(p));
        
    end
    
end    

[a,x] = min(BER1);
[b,y] = min(a);
        
        if b>1e-5
            disp('  ')
            disp('  ')
            EyePlotter(InputElectrical)
            disp('Eye closed!  Will stop further calculations.')
            disp('  ')
            disp('  ')
            return
        end
        
p_min = x(b==a); q_min = y; 

% Second, fine sweep of SampleTime & Threshold(Relative) near the sub-optimum value found by step #1

th_p = Threshold(p_min-1):0.2/20:Threshold(p_min+1);
Sa_p = SampleTime(q_min-1):0.2/20:SampleTime(q_min+1);

for m = 1:length(th_p);
    
    for n = 1:length(Sa_p);
        
         if nargout==5; % Q calculation

             [BER2(m,n),i_th2(m,n),Q2(m,n)] = qnber_ISI(InputElectrical,info_bit,Sa_p(n),th_p(m));
         
         elseif nargout < 5; % Q calculation is NOT requird.
            
             [BER2(m,n),i_th2(m,n)] = qnber_ISI(InputElectrical,info_bit,Sa_p(n),th_p(m));
             
         end
         
    end
    
end    


[a,x] = min(BER2);
[b,y] = min(a);
m_opt = x(b==a); n_opt = y;

BER_min = BER2(m_opt, n_opt);
RelativeSampleTime_opt = Sa_p(n_opt);
RelativeThreshold_opt = th_p(m_opt);
i_th_opt = i_th2(m_opt, n_opt);

  if nargout==5; % Q calculation
    Q_max = Q2(m_opt, n_opt);
  end


if strcmp(EyeDiagramOn, 'Yes') == 1

        EyePlotter2(InputElectrical,info_bit)
        hold on
        t_SampleTime = RelativeSampleTime_opt*max(1e9/BitRate)*ones(1,20); % EyePlotter2 plots eye in [ns].
        plot(t_SampleTime,1e3*linspace(0,2*i_th_opt,20),'k','LineWidth',2); % Magnitude in [mA or mV] 
        t_eye = 1e9*(0:1/SampleRate:(1/BitRate - 1/SampleRate));% 1 bit periods in [ns] unit
        plot(t_eye, 1e3*i_th_opt*ones(1,length(t_eye)),'k','LineWidth',2); % Magnitude in [mA or mV] 
        
end   
    
if strcmp(BER3D, 'Yes') == 1    
       
        figure 
        meshc(SampleTime,Threshold,log10(BER1))
        hold on
        meshc(Sa_p,th_p,log10(BER2))
        xlabel('Relative Sampling Time to Bit Period','FontSize',12)
        ylabel('Relative Decision Level','FontSize',12)
        zlabel('log10(BER)','FontSize',12)
        
end    