function [BER,i_th,Q] = qnber_ISI(InputElectrical,info_bit,SampleTime,Threshold)
%
% Usage: [BER,i_th,Q] = qnber_ISI(InputElectrical,info_bit,SampleTime,Threshold)
%
% Purpose = This module is to calculate BER (i_th & Q) based on Gaussian assumption. (WITH considering ISI effects)
%         = This module can be used to find optimum values with 'QnBER_ISI_Optimum'
%
% Input1 (InputElectrical) = Single channel signal (photo-detected electrical signal), Size = [1 x DataLength]
% Input2 (info_bit) = Bit sequence sent at the transmitter
%
% Output1 (BER) = Estimated BER at the SampleTime
% Output2 (i_th) = Physical threshold value, i_th = ms0 + Threshold*(ms1-ms0) (Threshold = Relative value);  
% Output3 (Q) = Estimated Q-value at the SampleTime  
%
%==== Simulation Parameters ===============
%  SampleTime = 0.5 (default),  range = [0,1[, The normalized sampling instant relative to the bit frame.
%  Threshlold = Relative threshold value, Default = 0.5 
%
%  Note1) Q-value is defined by the relationship, BER = (1/2)*erfc(Q/sqrt(2)) ~ exp(-Q^2/2)/(Q*sqrt(2*pi).
%  Note2) Neighboring one bit at ecah sides has been included to consider ISI effect. 
%

%----------------------------------------------------------
% $ Revision: 1.0 $ Date: 2008/04/26 by Jong-Hyung Lee  
%   Revision 1.1 $ Date: 2008/10/24 by Jong-Hyung Lee
%          (New min BER = 1e-200   Old min BER = 1e-100)
%----------------------------------------------------------
%
% Reference 1. C.J. Anderson and J.A. Lyle, "Technique for evaluating system performance using Q in numerical
%              simulations exhibiting intersymbol interference", Electronics Letters Jan. 1994 vol.33. no1. pp71.-72
%
%======================================================================================================================
global samplesperbit Bits  

if nargin <4, Threshold = 0.5; end
if nargin <3, SampleTime = 0.5; end

marks = mean(info_bit)*Bits; spaces = Bits - marks; % Number of Marks and Spaces in Bits

%--------------------------------------------------------------------------
% 1. Generate the synchronized input pulse train
%--------------------------------------------------------------------------

[Syn_out ref_pulse] = Synchronizer(InputElectrical,info_bit);

%--------------------------------------------------------------------------
% 2. Calculate mean & std of Marks & Spaces respectively at the Sampling time 
%--------------------------------------------------------------------------

Rel_samplesperbit = round(SampleTime*samplesperbit); 
Sampled_Syn_out = Syn_out(Rel_samplesperbit+samplesperbit*(0:(Bits-1)));

marks_v = Syn_out(ref_pulse==1);
marks_c = marks_v(Rel_samplesperbit+samplesperbit*(0:(marks-1))); % Mark values at the SampleTime of the pulse train
spaces_v = Syn_out(ref_pulse==0);
spaces_c = spaces_v(Rel_samplesperbit+samplesperbit*(0:(spaces-1))); % Space values at the SampleTime of the pulse train

ms1 = mean(marks_c); % std1 = std(marks_c);
ms0 = mean(spaces_c);% std0 = std(spaces_c);   
     
%--------------------------------------------------------------------------
% 3. Define 8 different bit patterns to consider ISI
%--------------------------------------------------------------------------

info_bit_a = [info_bit(Bits) info_bit info_bit(1)]; %two ends bits added to consider ISI effect 

pattern1_1 = [0 1 0]; pattern1_2 = [0 1 1]; pattern1_3 = [1 1 0]; pattern1_4 = [1 1 1];
pattern0_1 = [0 0 0]; pattern0_2 = [0 0 1]; pattern0_3 = [1 0 0]; pattern0_4 = [1 0 1];

%--------------------------------------------------------------------------
% 4. Collect sampled data defined by SampleTime for each 8 bit patterns.
%--------------------------------------------------------------------------
s1_1 = []; s1_2 = []; s1_3 = []; s1_4 = [];
s0_1 = []; s0_2 = []; s0_3 = []; s0_4 = [];

for n = 1:Bits;  
  
  if info_bit_a(n:n+2) == pattern1_1
      
      s1_1 = [s1_1 Sampled_Syn_out(n)];
      
   elseif info_bit_a(n:n+2) == pattern1_2
      
      s1_2 = [s1_2 Sampled_Syn_out(n)];
      
   elseif info_bit_a(n:n+2) == pattern1_3
      
      s1_3 = [s1_3 Sampled_Syn_out(n)];
 
   elseif info_bit_a(n:n+2) == pattern1_4
      
      s1_4 = [s1_4 Sampled_Syn_out(n)];
     
   elseif info_bit_a(n:n+2) == pattern0_1
      
      s0_1 = [s0_1 Sampled_Syn_out(n)];
      
   elseif info_bit_a(n:n+2) == pattern0_2
      
      s0_2 = [s0_2 Sampled_Syn_out(n)];

   elseif info_bit_a(n:n+2) == pattern0_3
      
      s0_3 = [s0_3 Sampled_Syn_out(n)];
      
   elseif info_bit_a(n:n+2) == pattern0_4
      
      s0_4 = [s0_4 Sampled_Syn_out(n)];
      
   end
   
end   
 
%--------------------------------------------------------------------------
% 5. Mean, Standard Deviation, and the frequency of occurence of each pattern
%--------------------------------------------------------------------------

np1_1 = length(s1_1); np1_2 = length(s1_2); np1_3 = length(s1_3); np1_4 = length(s1_4);
np0_1 = length(s0_1); np0_2 = length(s0_2); np0_3 = length(s0_3); np0_4 = length(s0_4);

np = [np1_1 np1_2 np1_3 np1_4 np0_1 np0_2 np0_3 np0_4];

if any(np < 20)
    
    disp('   ')
    disp('   ')
    disp('The number of each bit pattern occurance is too small to be statistically meaningful.')
    disp('Increase the number of Bits')
    disp('   ')
    return
    
else     

    ms1_1 = mean(s1_1); ms1_2 = mean(s1_2); ms1_3 = mean(s1_3); ms1_4 = mean(s1_4);
    std1_1 = std(s1_1); std1_2 = std(s1_2); std1_3 = std(s1_3); std1_4 = std(s1_4); 

    ms0_1 = mean(s0_1); ms0_2 = mean(s0_2); ms0_3 = mean(s0_3); ms0_4 = mean(s0_4);
    std0_1 = std(s0_1); std0_2 = std(s0_2); std0_3 = std(s0_3); std0_4 = std(s0_4); 

end


%--------------------------------------------------------------------------
% 6. Calculation of Q & BER  
%--------------------------------------------------------------------------


% i_th (Decision Level) = ms0 + Threshold*(ms1-ms0);.   
   
   
   i_th = ms0 + Threshold*(ms1-ms0); 
   Ber1_1 = erfc(abs(ms1_1 - i_th)/sqrt(2)/std1_1);
   Ber1_2 = erfc(abs(ms1_2 - i_th)/sqrt(2)/std1_2);
   Ber1_3 = erfc(abs(ms1_3 - i_th)/sqrt(2)/std1_3);
   Ber1_4 = erfc(abs(ms1_4 - i_th)/sqrt(2)/std1_4);
   
   Ber0_1 = erfc(abs(ms0_1 - i_th)/sqrt(2)/std0_1);
   Ber0_2 = erfc(abs(ms0_2 - i_th)/sqrt(2)/std0_2);
   Ber0_3 = erfc(abs(ms0_3 - i_th)/sqrt(2)/std0_3);
   Ber0_4 = erfc(abs(ms0_4 - i_th)/sqrt(2)/std0_4);
   
   BER = (np1_1*Ber1_1+np1_2*Ber1_2+np1_3*Ber1_3+np1_4*Ber1_4+...
      np0_1*Ber0_1+np0_2*Ber0_2+np0_3*Ber0_3+np0_4*Ber0_4)/sum(np); % sum(np) = np1_1 + np1_2 + ..... = Bits
   
             if BER < 1e-200
                BER = 1e-200;
             end   

if nargout==3; % Q calculation
                 
   log_ber = log(BER);%  Note that it is not 'log10', but 'log' (natural logarithm)
   save ber_sim log_ber;% To be used in 'Q_value' function to calculate corresponding Q value.
   
   if log10(BER) > -9
      Q_ini = 6;           % initial assumption of Q value
   elseif log10(BER) > -50
      Q_ini = 15;
   else
      Q_ini = 20;
   end
   
   options = optimset('Display','off');% Suppress displaying calculation process by 'fzero'
   Q = fzero('Q_value',Q_ini,options); % 
   delete('ber_sim.mat');
   
end
    
